/* ==========================================================================*/
/* - - - - - - - - - - - -  M E N U , C  - - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   07-10-2004   M. Wheatley                                       */
/*	Modified  08-28-2004   MW                                                */
/*.........................................................................  */
/* MENU Module                                                               */
/*.........................................................................  */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include <p30f6014.h>
#include <spi.h>
#include <dsp.h>
#include "common.h"
#include "timers.h"
#include "procsignal.h"
#include "lcd.h"
#include "atod.h"
#include "codec.h"
#include "gettime.h"
#include "soundgen.h"
#include "menu.h"


/*---------------------------------------------------------------------------*/
/*----------------> P R O C E S S O R    S P E C I F I C <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/

#define M1ITEM1 0
#define M1ITEM2 1
#define M1ITEM3 2
#define M1ITEM4 3

/*---------------------------------------------------------------------------*/
/*----------------> E X T E R N A L  R E F E R E N C E S <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------> G L O B A L  M E M O R Y  A L L O C A T I O N <------------*/
/*---------------------------------------------------------------------------*/
unsigned char CurrentMenu;

/*---------------------------------------------------------------------------*/
/*--------------> L O C A L  M E M O R Y  A L L O C A T I O N <--------------*/
/*---------------------------------------------------------------------------*/
static const char SPLASH1_STR[] = "Cuckoo Clock Program";
static const char SPLASH2_STR[] = "     MoeTronix";
static const char SPLASH3_STR[] = " www.moetronix.com";
static const char SPLASH4_STR[] = "V1.0    Aug 29, 2004";

static const char AM_STR[] = " AM";
static const char PM_STR[] = " PM";

static const char OFFSET_STR[] = "Local Hr Offset=";
static const char STATION_STR[] = "Station =";
static const char WWV_STR[] = "WWV ";
static const char WWVB_STR[] = "WWVB";
static const char CHIME_STR[] = "Hour Chime =";
static const char GONG_STR[] = "GONG  ";
static const char CUCKOO_STR[] = "CUCKOO";
static const char TICKTOCK_STR[] = "Tic-Tock =";
static const char ON_STR[] = "ON ";
static const char OFF_STR[] = "OFF";

static const char SYNC_STR[] =     "Sync     Min Qual";
static const char MINENRGY_STR[] = "Min Energy";
static const char HRENRGY_STR[] =  "Hr Energy";
static const char YRENRGY_STR[] =  "Yr Energy";

static const char MANSET_STR[] =  "   Manual Time Set  ";
static const char YEARSET_STR[] = "Year = ";
static const char HOURSET_STR[] = "UTC Hour = ";
static const char MINSET_STR[] = "Minute = ";

static unsigned int CurrentPot1;
static unsigned int CurrentPot2;
static unsigned int CurrentPot3;

static char ItemSelect;
/*---------------------------------------------------------------------------*/
/*-----------------------> P R O T O T Y P E S <-----------------------------*/
/*---------------------------------------------------------------------------*/
void InitMenu(void);
void ServiceMenu(void);
void DisplayTime( void);

static void MenuScroll(void);
static void ItemScroll(void);
static void ItemChange(char val);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             -----------------                             */
/*                            | I n i t M e n u |                            */
/*                             -----------------                             */
/*    This initializes the Menu System.                                      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitMenu(void)
{
unsigned int tmp;
	SetLcdPos(0,0);
	WriteCStr(SPLASH1_STR);
	SetLcdPos(0,1);
	WriteCStr(SPLASH2_STR);
	SetLcdPos(0,2);
	WriteCStr(SPLASH3_STR);
	SetLcdPos(0,3);
	WriteCStr(SPLASH4_STR);
	TimeDelay(50);
	ClearLcd();
	//Read in Non-volatile program settings
	CurrentMenu = (unsigned char)ReadEEData( EE_MENU);
	CurrentMenu--;
	if(CurrentMenu > MENU_DATA)
		CurrentMenu = MENU_DATA;
	YearCount = (unsigned char)ReadEEData( EE_MANUALYEAR );
	if(YearCount>20)
		YearCount = 4;
	if(YearCount<4)
		YearCount = 20;
	HoursCount = (unsigned char)ReadEEData( EE_MANUALHOUR );
	if((char)HoursCount>23)
		HoursCount = 0;
	if( (char)HoursCount<0)
		HoursCount = 23;
	MinutesCount = (unsigned char)ReadEEData( EE_MANUALMIN );
	if((char)MinutesCount>59)
		MinutesCount = 0;
	if((char)MinutesCount<0)
		MinutesCount = 59;

	LocalHoursOffset = (char)ReadEEData( EE_LOCALOFFSET);
	tmp = ReadEEData( EE_WWVSEL);
	if(tmp)
		SemB.WWV = 1;
	else
		SemB.WWV = 0;
	tmp = ReadEEData( EE_TICKTOCKSEL);
	if(tmp)
		SemB.TicTock = 1;
	else
		SemB.TicTock = 0;
	tmp = ReadEEData( EE_CUCKOOSEL);
	if(tmp)
		SemB.Cuckoo = 1;
	else
		SemB.Cuckoo = 0;
	KeyNum = KEY1;
	SemA.KeySwitch = TRUE;	//fake key press to intialize screen to menu
	CurrentPot1 = 0;
	CurrentPot2 = 0;
	CurrentPot3 = 0;
	SemA.PlotActive = FALSE;
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        -----------------------                            */
/*                       | S e r v i c e M e n u |                           */
/*                        -----------------------                            */
/*    Called in main loop to manage Menu system.                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ServiceMenu( )
{
int tmp;
	if(SemA.KeySwitch && !SemA.PlotActive)	//if key pressed and not drawing
	{
		SemA.KeySwitch = FALSE;
		switch(KeyNum)
		{
			case KEY1:		//Menu scroll key
				MenuScroll();
				break;
			case KEY2:		//Menu Item scroll key
				ItemScroll();
				break;
			case KEY3:		//decrement item
				ItemChange(-1);
				break;
			case KEY4:		//increment item
				ItemChange(1);
				break;
		}
	}
	if(SemA.Timer1)	//Update Pot values every 100mS
	{
		SemA.Timer1 = 0;
		tmp = (GetADValue(POT1)>>5) & 0x00FC;
		if(	CurrentPot1 != tmp )
		{
			CurrentPot1 = tmp;
			// send new input volume settings to CODEC
			WriteCodecCtrl( CODEC_ADCVOL, tmp|CODEC_PARM_LOM ); 
		}
		tmp = (GetADValue(POT3)>>5) & 0x00FC;
		if(	CurrentPot3 != tmp )
		{
			CurrentPot3 = tmp;
			if(tmp > 0x5C)	//dont let the gain go above 0dB
				tmp = 0x5C;
			// send new output volume settings to CODEC
			WriteCodecCtrl( CODEC_DACVOL, tmp|CODEC_PARM_SLM|CODEC_PARM_SRM );
		}
		tmp = GetADValue(POT2);
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        ---------------------                              */
/*                       | M e n u S c r o l l |                             */
/*                        ---------------------                              */
/*    Called when menu scroll key is pressed to increment to the next menu.  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void MenuScroll(void)
{
	CurrentMenu++;
	if(CurrentMenu > MENU_DATA)
		CurrentMenu = MENU_MAIN;
	ClearLcd();
	switch(CurrentMenu)
	{
		case MENU_MAIN:		//Clock View Menu
			CursState(0);
			SetLcdPos(8,0);		//draw fixed text
			WriteChar(':');
			SetLcdPos(11,0);
			WriteChar(':');
			SetLcdPos(15,0);
			WriteChar('U');	WriteChar('T');	WriteChar('C');
			SetLcdPos(8,1);
			WriteChar(':');
			SetLcdPos(11,1);
			WriteChar(':');
			SetLcdPos(8,2);
			WriteChar('2');	WriteChar('0');
			SemB.SoundInhibit = FALSE;
			break;
		case MENU_SETUP:	//Setup Menu
			SetLcdPos(0,3);
			WriteCStr(TICKTOCK_STR);
			ItemSelect = M1ITEM4;
			ItemChange(0);
			SetLcdPos(0,2);
			WriteCStr(CHIME_STR);
			ItemSelect = M1ITEM3;
			ItemChange(0);
			SetLcdPos(0,1);
			WriteCStr(STATION_STR);
			ItemSelect = M1ITEM2;
			ItemChange(0);
			SetLcdPos(0,0);		//draw fixed text
			WriteCStr(OFFSET_STR);
			ItemSelect = M1ITEM1;
			ItemChange(0);
			CursState(1);
			SemB.SoundInhibit = FALSE;
			break;
		case MENU_STATUS:	//Acq status menu
			CursState(0);
			SetLcdPos(0,0);		//draw fixed text
			WriteCStr(SYNC_STR);
			SetLcdPos(0,1);
			WriteCStr(MINENRGY_STR);
			SetLcdPos(0,2);
			WriteCStr(HRENRGY_STR);
			SetLcdPos(0,3);
			WriteCStr(YRENRGY_STR);
			SemB.SoundInhibit = FALSE;
			break;
		case MENU_SETTIME:
			SemB.SoundInhibit = TRUE;
			SetLcdPos(0,0);
			WriteCStr(MANSET_STR);
			SetLcdPos(0,3);
			WriteCStr(MINSET_STR);
			ItemSelect = M1ITEM3;
			ItemChange(0);
			SetLcdPos(0,2);
			WriteCStr(HOURSET_STR);
			ItemSelect = M1ITEM2;
			ItemChange(0);
			SetLcdPos(0,1);
			WriteCStr(YEARSET_STR);
			ItemSelect = M1ITEM1;
			ItemChange(0);
			CursState(1);
			break;
		case MENU_SIGNAL:	//signal input plot
			SemB.SoundInhibit = FALSE;
			CursState(0);
			WriteEEData( EE_MANUALMIN , (int)MinutesCount);
			WriteEEData( EE_MANUALHOUR , (int)HoursCount);
			WriteEEData( EE_MANUALYEAR , (int)YearCount);
			break;
		case MENU_SYNC:		//Sec sync signal plot
		case MENU_DATA:		//data signal plot
			SemB.SoundInhibit = FALSE;
			CursState(0);
			break;
	}
	WriteEEData( EE_MENU , (int)CurrentMenu);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        ---------------------                              */
/*                       | I t e m S c r o l l |                             */
/*                        ---------------------                              */
/*    Called when item scroll key is pressed to increment to the next item.  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ItemScroll(void)
{
	switch(CurrentMenu)
	{
		case MENU_SETUP:	//Setup Menu
			ItemSelect++;
			if(	ItemSelect > M1ITEM4)
				ItemSelect = M1ITEM1;
			ItemChange(0);
			break;
		case MENU_SETTIME:
			ItemSelect++;
			if(	ItemSelect > M1ITEM3)
				ItemSelect = M1ITEM1;
			ItemChange(0);
			break;
		case MENU_MAIN:		//Clock View Menu
		case MENU_STATUS:	//Acq status menu
		case MENU_SIGNAL:	//signal input plot
		case MENU_SYNC:		//Sec sync signal plot
		case MENU_DATA:		//data signal plot
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        ---------------------                              */
/*                       | I t e m C h a n g e |                             */
/*                        ---------------------                              */
/*    Called when item inc/dec key is pressed to change an Item.             */
/*    if val == 0 is an initialization call                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ItemChange(char val)
{
	if(	CurrentMenu == MENU_SETUP)
	{
		switch(ItemSelect)
		{
			case M1ITEM1:	//Local time offset
				LocalHoursOffset = (char)LocalHoursOffset + val;
				if(LocalHoursOffset>12)
					LocalHoursOffset = 12;
				if(LocalHoursOffset<-12)
					LocalHoursOffset = -12;
				SetLcdPos(17,0);
				if(LocalHoursOffset>=0)
				{
					WriteChar('+');
					Write2Dec(LocalHoursOffset);
				}
				else
				{
					WriteChar('-');
					Write2Dec(-LocalHoursOffset);
				}
				if( val!=0 )
					WriteEEData( EE_LOCALOFFSET , (int)LocalHoursOffset);
				SetLcdPos(19,0);
				break;
			case M1ITEM2:	//Select WWV or WWVB station
				SetLcdPos(10,1);
				if(0 != val)
				{
					SemB.WWV = !SemB.WWV;	//toggle station
					if(SemB.WWV)
						WriteEEData( EE_WWVSEL , 1);
					else
						WriteEEData( EE_WWVSEL , 0);
					InitProcSignal();	//need to reset DSP filters
				}
				if(SemB.WWV)
					WriteCStr(WWV_STR);
				else
					WriteCStr(WWVB_STR);
				break;
			case M1ITEM3:	//Select Gong or Cuckoo hourly chime
				SetLcdPos(13,2);
				if(0 != val)
				{
					SemB.Cuckoo = !SemB.Cuckoo;	//toggle 
					if(SemB.Cuckoo)
						WriteEEData( EE_CUCKOOSEL , 1);
					else
						WriteEEData( EE_CUCKOOSEL , 0);
				}
				if(SemB.Cuckoo)
					WriteCStr(CUCKOO_STR);
				else
					WriteCStr(GONG_STR);
				break;
			case M1ITEM4:	//Select Tick Tock ON or OFF
				SetLcdPos(11,3);
				if(0 != val)
				{
					SemB.TicTock = !SemB.TicTock;	//toggle 
					if(SemB.TicTock)
						WriteEEData( EE_TICKTOCKSEL , 1);
					else
						WriteEEData( EE_TICKTOCKSEL , 0);
				}
				if(SemB.TicTock)
					WriteCStr(ON_STR);
				else
					WriteCStr(OFF_STR);
				break;
		}
	}
	else if( CurrentMenu == MENU_SETTIME)
	{
		switch(ItemSelect)
		{
			case M1ITEM1:	//Last 2 digits of year
				YearCount = (char)YearCount + val;
				if(YearCount>20)
					YearCount = 4;
				if(YearCount<4)
					YearCount = 20;
				SetLcdPos(7,1);
				Write2Dec(YearCount);
				SetLcdPos(8,1);
				if( 0 != val )
					WriteEEData( EE_MANUALYEAR , (int)YearCount);
				break;
			case M1ITEM2:	//UTC hours
				HoursCount = (char)HoursCount + val;
				if((char)HoursCount>23)
					HoursCount = 0;
				if( (char)HoursCount<0)
					HoursCount = 23;
				SetLcdPos(11,2);
				Write2Dec(HoursCount);
				SetLcdPos(12,2);
				if( 0 != val )
					WriteEEData( EE_MANUALHOUR , (int)HoursCount);
				break;
			case M1ITEM3:	//Minutes
				MinutesCount = (char)MinutesCount + val;
				if((char)MinutesCount>59)
					MinutesCount = 0;
				if((char)MinutesCount<0)
					MinutesCount = 59;
				SetLcdPos(11,3);
				Write2Dec(MinutesCount);
				SetLcdPos(12,3);
				if( val!=0 )
				{
					WriteEEData( EE_MANUALMIN , (int)MinutesCount);
					SecondsCount = 0;
				}
				break;
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | D i s p l a y T i m e |                            */
/*                       -----------------------                             */
/* Display current time values on LCD                  				         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void DisplayTime(void)
{
	switch(CurrentMenu)
	{
		case MENU_MAIN:		//Clock View Menu
			SetLcdPos(6,0);
			Write2Dec(HoursCount);
			SetLcdPos(9,0);
			Write2Dec(MinutesCount);
			SetLcdPos(12,0);
			Write2Dec(SecondsCount);
			SetLcdPos(6,1);
			Write2Dec(LocalHoursCount);
			SetLcdPos(9,1);
			Write2Dec(MinutesCount);
			SetLcdPos(12,1);
			Write2Dec(SecondsCount);
			if(SemA.PMAM)
				WriteCStr(PM_STR);
			else
				WriteCStr(AM_STR);
			SetLcdPos(10,2);
			Write2Dec(YearCount);
			break;
		case MENU_SETUP:	//Setup Menu
			break;
		case MENU_STATUS:	//Acq status menu
			SetLcdPos(5,0);
			if(SemA.SecLocked)
				WriteChar('Y');
			else
				WriteChar('N');
			SetLcdPos(18,0);
			Write2Dec(MinSyncQuality);
			SetLcdPos(11,1);
			Write2Dec(MinEnergy);
			SetLcdPos(11,2);
			Write2Dec(HrEnergy);
			SetLcdPos(11,3);
			Write2Dec(YrEnergy);
			break;
		case MENU_SIGNAL:	//signal input plot
			break;
		case MENU_SYNC:		//Sec sync signal plot
			break;
		case MENU_DATA:		//data signal plot
			break;
	}
}
